cclog = function(...)
    CCLuaLog(string.format(...))
end

-- for CCLuaEngine traceback
function __G__TRACKBACK__(msg)
    cclog("----------------------------------------")
    cclog("LUA ERROR: " .. tostring(msg) .. "\n")
    cclog(debug.traceback())
    cclog("----------------------------------------")
end

function string.split(input, delimiter)  
    input = tostring(input)  
    delimiter = tostring(delimiter)  
    if (delimiter=='') then return false end  
    local pos,arr = 0, {}  
    -- for each divider found  
    for st,sp in function() return string.find(input, delimiter, pos, true) end do  
        table.insert(arr, string.sub(input, pos, st - 1))  
        pos = sp + 1  
    end  
    table.insert(arr, string.sub(input, pos))  
    return arr  
end

function string.trim(input)
    return string.gsub(input, "^%s*(.-)%s*$", "%1")
end

function dump(value, desciption, nesting)  
    if type(nesting) ~= "number" then nesting = 3 end  
  
    local lookupTable = {}  
    local result = {}  
  
    local function _v(v)  
        if type(v) == "string" then  
            v = "\"" .. v .. "\""  
        end  
        return tostring(v)  
    end  
  
    local traceback = string.split(debug.traceback("", 2), "\n")  
    print("dump from: " .. string.trim(traceback[3]))  
  
    local function _dump(value, desciption, indent, nest, keylen)  
        desciption = desciption or "<var>"  
        spc = ""  
        if type(keylen) == "number" then  
            spc = string.rep(" ", keylen - string.len(_v(desciption)))  
        end  
        if type(value) ~= "table" then  
            result[#result +1 ] = string.format("%s%s%s = %s", indent, _v(desciption), spc, _v(value))  
        elseif lookupTable[value] then  
            result[#result +1 ] = string.format("%s%s%s = *REF*", indent, desciption, spc)  
        else  
            lookupTable[value] = true  
            if nest > nesting then  
                result[#result +1 ] = string.format("%s%s = *MAX NESTING*", indent, desciption)  
            else  
                result[#result +1 ] = string.format("%s%s = {", indent, _v(desciption))  
                local indent2 = indent.."    "  
                local keys = {}  
                local keylen = 0  
                local values = {}  
                for k, v in pairs(value) do  
                    keys[#keys + 1] = k  
                    local vk = _v(k)  
                    local vkl = string.len(vk)  
                    if vkl > keylen then keylen = vkl end  
                    values[k] = v  
                end  
                table.sort(keys, function(a, b)  
                    if type(a) == "number" and type(b) == "number" then  
                        return a < b  
                    else  
                        return tostring(a) < tostring(b)  
                    end  
                end)  
                for i, k in ipairs(keys) do  
                    _dump(values[k], k, indent2, nest + 1, keylen)  
                end  
                result[#result +1] = string.format("%s}", indent)  
            end  
        end  
    end  
    _dump(value, desciption, "- ", 1)  
  
    for i, line in ipairs(result) do  
        print(line)  
    end  
end  

--封装函数
function handler(obj, method)
    return function(...)
       return method(obj, ...)
    end
end

--数字变化
local _Action_Change_Num_Tag = 33424
function labelChange2(label, num, time)
    if label.curNum == nil then
        label.curNum = num
        label:setText(num)
    else
        label:stopActionByTag(_Action_Change_Num_Tag)
        if label.curNum ~= num then
            label.numStep = math.ceil((num - label.curNum) / time / 10)
            local action = CCRepeatForever:create(CCSequence:createWithTwoActions(CCDelayTime:create(0.1), CCCallFuncN:create(function(node)
                local nextNum = label.curNum
                if label.numStep > 0 then
                    nextNum = math.min(num, label.curNum + label.numStep + math.random(0, label.numStep))
                elseif label.numStep < 0 then
                    nextNum = math.max(num, label.curNum + label.numStep + math.random(label.numStep, 0))
                end
                label.curNum = nextNum
                label:setText(label.curNum)
                if nextNum == num then
                    label:stopActionByTag(_Action_Change_Num_Tag)
                end
            end)))

            action:setTag(_Action_Change_Num_Tag)
            label:runAction(action)
        end
    end
end

--弧度与角度转换函数
function degrees2radians(angle)
    return angle * 0.01745329252
end

function radians2degrees(angle)
    return angle * 57.29577951
end

--设置字体描边
function labelEnableStroke(uiLable, strokeColor, strokeSize)
	if cc.TargetPlatform ~= cc.TargetPlatformType.Windows then
		local label = tolua.cast(uiLable:getVirtualRenderer(), "CCLabelTTF")
		if label.enableStroke then
			label:enableStroke(strokeColor, strokeSize, true)
		end
	end 
end

--设置字体阴影
function labelEnableShadow(shadowOffset, shadowOpacity, shadowBlur)
    if cc.TargetPlatform ~= cc.TargetPlatformType.Windows then
        local label = tolua.cast(uiLable:getVirtualRenderer(), "CCLabelTTF")
        if label.enableShadow then
            label:enableShadow(shadowOffset, shadowOpacity, shadowBlur, true)
        end
    end  
end
